@@ -31,6 +31,7 @@ import string
 import subprocess
 import time
 import uuid
+from inspect import signature
 from pathlib import Path
 from subprocess import PIPE, Popen
 from typing import Any, Dict, List, Optional, Sequence, Union
@@ -498,7 +499,12 @@ class CloudSqlProxyRunner(LoggingMixin):
             )
         proxy_path_tmp = self.sql_proxy_path + ".tmp"
         self.log.info("Downloading cloud_sql_proxy from %s to %s", download_url, proxy_path_tmp)
-        response = httpx.get(download_url, allow_redirects=True)
+        # httpx has a breaking API change (follow_redirects vs allow_redirects)
+        # and this should work with both versions (cf. issue #20088)
+        if "follow_redirects" in signature(httpx.get).parameters.keys():
+            response = httpx.get(download_url, follow_redirects=True)
+        else:
+            response = httpx.get(download_url, allow_redirects=True)
         # Downloading to .tmp file first to avoid case where partially downloaded
         # binary is used by parallel operator which uses the same fixed binary path
         with open(proxy_path_tmp, 'wb') as file:
@@ -768,7 +774,7 @@ class CloudSQLDatabaseHook(BaseHook):
 
     @staticmethod
     def _get_bool(val: Any) -> bool:
-        if val == 'False':
+        if val == 'False' or val is False:
             return False
         return True
 
