@@ -18,8 +18,16 @@
 # shellcheck source=scripts/ci/libraries/_script_init.sh
 . "$( dirname "${BASH_SOURCE[0]}" )/../libraries/_script_init.sh"
 
+# Pushes Ci images with tags to registry in GitHub
+function push_ci_image_with_tag_to_github() {
+    start_end::group_start "Push CI image"
+    docker_v tag "${AIRFLOW_CI_IMAGE}" "${AIRFLOW_CI_IMAGE}:${GITHUB_REGISTRY_PUSH_IMAGE_TAG}"
+    push_pull_remove_images::push_image_with_retries "${AIRFLOW_CI_IMAGE}:${GITHUB_REGISTRY_PUSH_IMAGE_TAG}"
+    start_end::group_end
+}
+
 build_images::prepare_ci_build
 
 build_images::login_to_docker_registry
 
-push_pull_remove_images::push_ci_images_to_github
+push_ci_image_with_tag_to_github
