@@ -18,8 +18,17 @@
 # shellcheck source=scripts/ci/libraries/_script_init.sh
 . "$( dirname "${BASH_SOURCE[0]}" )/../libraries/_script_init.sh"
 
+# Pushes PROD images with tags to registry in GitHub
+function push_prod_image_with_tag_to_github () {
+    start_end::group_start "Push PROD image"
+    local airflow_prod_tagged_image="${AIRFLOW_PROD_IMAGE}:${GITHUB_REGISTRY_PUSH_IMAGE_TAG}"
+    docker_v tag "${AIRFLOW_PROD_IMAGE}" "${airflow_prod_tagged_image}"
+    push_pull_remove_images::push_image_with_retries "${airflow_prod_tagged_image}"
+    start_end::group_end
+}
+
 build_images::prepare_prod_build
 
 build_images::login_to_docker_registry
 
-push_pull_remove_images::push_prod_images_to_github
+push_prod_image_with_tag_to_github
