@@ -18,11 +18,7 @@
 # shellcheck source=scripts/ci/libraries/_script_init.sh
 . "$( dirname "${BASH_SOURCE[0]}" )/../libraries/_script_init.sh"
 
-EXECUTOR=KubernetesExecutor
-export EXECUTOR
-
-# We started with KubernetesExecutor. Let's run tests first
-"$( dirname "${BASH_SOURCE[0]}" )/ci_run_kubernetes_tests.sh"
+# There is no need to run tests before upgrade (other tests do that). Let's test it after.
 for EXECUTOR in CeleryExecutor KubernetesExecutor
 do
     kind::upgrade_airflow_with_helm "${EXECUTOR}"
